﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Service.BoardService;
using Service;
using System.Collections.ObjectModel;
using com.google.zxing.common;
using com.google.zxing.qrcode;
using System.Drawing;
using System.Windows.Interop;
using Common;
using BasisKlassen;
using Common.Logger;

namespace AnzeigenApp
{
    /// <summary>
    /// Interaktionslogik für AnzeigeDetailView.xaml
    /// </summary>
    public partial class AnzeigeDetailView : MyUserControl
    {
        private string guid;
        private Anzeige anzeige;
        private FilterAnzeigenView filterView;

        public AnzeigeDetailView(Anzeige anzeige, string guid, FilterAnzeigenView filterView)
        {
            Logger.LogEvent(LogEvent.InseratAngeschaut, filterView.Model.Guid);
            InitializeComponent();
            images = new ObservableCollection<ImageSource>();
            this.anzeige = anzeige;
            DataContext = this;
            this.Guid = guid;
            this.filterView = filterView;
            UpdateData(anzeige);
        }

        public string Guid
        {
            get { return guid; }
            set
            {
                if (Guid != value)
                {
                    guid = value;
                    this.OnPropertyChanged("Guid");
                }
            }
        }

        private ObservableCollection<ImageSource> images;

        public ObservableCollection<ImageSource> Images
        {
            get
            {
                return images;
            }
            set
            {
                if (images != value)
                {
                    images = value;
                    //this.OnPropertyChanged("Images");
                }
            }
        }

        private void UpdateData(Anzeige anzeige)
        {
            if (anzeige != null)
            {
                if (String.IsNullOrEmpty(guid))
                    gridSendPerMail.Visibility = Visibility.Collapsed;
                else
                    gridSendPerMail.Visibility = Visibility.Visible;

                int maxSize = Common.AppConfHelper.GetValueAsInt("max_qr_code_size");
                images.Clear();
                txtText.Text = anzeige.Text;
                String tmp = String.Empty;
                foreach (var s in anzeige.Tags)
                    tmp += s + ", ";
                if (tmp.Length >= 2)
                    tmp = tmp.Substring(0, tmp.Length - 2);
                txtTags.Text = tmp;
                txtTitel.Text = anzeige.Titel;
                txtBeschreibung.Text = anzeige.Beschreibung;
                //string tmpKategorie = "";
                //foreach (KeyValuePair<int, string> kvp in anzeige.Kategorie)
                //    tmpKategorie += kvp.Value + "\n";
                //textBlock3.Text = tmpKategorie;
                foreach (String path in anzeige.Images)
                    images.Add(BitmapLoader.GetImageSource(path));
                if (images.Count() > 0)
                {
                    imgPreview.Source = images.First();
                    txtImgInfo.Visibility = Visibility.Visible;
                }
                //Image img1 = new Image();
                //Image img2 = new Image();
                //Image img3 = new Image();
                //img1.Source = src1;
                //img2.Source = src1;
                //img3.Source = src1;

                pics.ItemsSource = Images;

                String data = anzeige.ToJsonString(maxSize == 0 ? 300 : maxSize);
                ByteMatrix matrix;
                com.google.zxing.Writer writer = new QRCodeWriter();
                try
                {
                    matrix = writer.encode(data, com.google.zxing.BarcodeFormat.QR_CODE, 400, 400);
                    int width = matrix.Width;
                    int height = matrix.Height;

                    sbyte[][] array = matrix.Array;
                    Bitmap image = new Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
                    for (int y = 0; y < height; y++)
                    {
                        for (int x = 0; x < width; x++)
                        {
                            int grayValue = array[y][x] & 0xff;
                            image.SetPixel(x, y, grayValue == 0 ? System.Drawing.Color.Black : System.Drawing.Color.White);
                        }
                    }
                    var src = Imaging.CreateBitmapSourceFromHBitmap(image.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromWidthAndHeight(width, height));
                    qrCode.Source = src;
                    //images.Add(src);
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);
                }
            }
        }

        private void SurfaceButton_Click(object sender, RoutedEventArgs e)
        {
            Proxy fassade = new Proxy();
            fassade.AnzeigeLöschen(guid, anzeige.ID);
            filterView.UnfilteredAnzeigen.Remove(anzeige);
            filterView.SelectedAnzeige = null;
        }

        //private void SurfaceButton_Click_1(object sender, RoutedEventArgs e)
        //{
        //    Proxy fassade = new Proxy();
        //    fassade.AnzeigeSetÖffentlichStatus(guid, anzeige.ID, !anzeige.Published);
        //    Anzeige newAnzeige = fassade.GetAnzeige(guid, anzeige.ID);
        //    if (anzeige.Equals(newAnzeige))
        //        if (newAnzeige != null)
        //        {
        //            this.anzeige = newAnzeige;
        //            UpdateData(newAnzeige);
        //            filterView.UnfilteredAnzeigen.Where(x => x.ID == anzeige.ID).Single().UpdateValues(anzeige);
        //            //filterView.UnfilteredAnzeigen.Remove(anzeige);
        //            //filterView.UnfilteredAnzeigen.Add(newAnzeige);
        //        }
        //}

        private void SurfaceButton_Click_2(object sender, RoutedEventArgs e)
        {
            Logger.LogEvent(LogEvent.InseratPerMailGesendet, filterView.Model.Guid);
            Proxy fassade = new Proxy();
            if (fassade.SendeAnzeigePerMail(guid, anzeige.ID))
                gridSendPerMail.Visibility = Visibility.Hidden;
            else
                txtMailError.Visibility = Visibility.Visible;
        }

        private void SurfaceButton_Click_3(object sender, RoutedEventArgs e)
        {
            gridImages.Visibility = Visibility.Hidden;
        }

        private void imgPreview_TouchDown(object sender, TouchEventArgs e)
        {
            gridImages.Visibility = Visibility.Visible;
        }

        private void btnShowProfile_Click_1(object sender, RoutedEventArgs e)
        {
            profileGrid.Visibility = Visibility.Visible;
            try
            {
                var proxy = new Proxy();
                Profil profil = proxy.GetProfilByID(guid, anzeige.AccountID);
                profileContent.update(profil);
                //profileContent = new ProfilApp.ProfilDetailView(profil);
            }
            catch (Exception ex) { }
            //TextBlock txt = new TextBlock();
            //txt.Text = "böuib";
            //profileContent.Content = txt;
        }

        private void SurfaceButton_Click_1(object sender, RoutedEventArgs e)
        {
            profileGrid.Visibility = Visibility.Hidden;
        }

        private void UserControl_Loaded_1(object sender, RoutedEventArgs e)
        {
            profileGrid.Visibility = Visibility.Hidden;
            //gridSendPerMail.Visibility = Visibility.Visible;
            txtMailError.Visibility = Visibility.Collapsed;
        }
    }
}
