﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FluentNHibernate.Mapping;
using System.Runtime.Serialization;
using ProfilKomponente;
using System.IO;

namespace AnzeigenKomponente
{
    [DataContract(Namespace = "http://marius-klausa.eu/board")]
    public class TransportAnzeige
    {
        [DataMember]
        public virtual int ID { get; private set; }
        [DataMember]
        public virtual int AccountID { get; private set; }
        [DataMember]
        public virtual string profilNickname { get; private set; }
        [DataMember]
        public virtual String Titel { get; set; }
        [DataMember]
        public virtual String Beschreibung { get; set; }
        [DataMember]
        public virtual String Text { get; set; }
        [DataMember]
        public virtual DateTime Datum { get; set; }
        [DataMember]
        public virtual TimeSpan Restzeit { get; set; }
        [DataMember]
        public virtual IList<string> Tags { get; set; }
        [DataMember]
        public virtual IDictionary<int, string> Kategorie { get; set; }
        [DataMember]
        public virtual string Appearance { get; set; }
        [DataMember]
        public virtual IList<string> Images { get; set; }
        [DataMember]
        public virtual bool Published { get; set; }
        [DataMember]
        public virtual KontaktDetails KontaktDetails { get; set; }

        public TransportAnzeige(Anzeige anzeige, string nickname, IList<string> images, KontaktDetails kontaktDetails)
        {
            this.Titel = anzeige.Titel;
            this.ID = anzeige.ID;
            this.AccountID = anzeige.AccountID;
            this.profilNickname = nickname;
            this.Beschreibung = anzeige.Beschreibung;
            this.Text = anzeige.Text;
            this.Datum = anzeige.Datum;
            this.Restzeit = anzeige.Restzeit;
            this.Published = anzeige.Published;
            this.Tags = new List<string>();
            foreach (Tag t in anzeige.Tags)
                this.Tags.Add(t.Name);
            this.Kategorie = new Dictionary<int, string>();
            foreach (KeyValuePair<int, Kategorie> kvp in anzeige.Kategorie)
                this.Kategorie.Add(new KeyValuePair<int, string>(kvp.Key, kvp.Value.Name));
            this.Appearance = anzeige.Appearance;
            this.Images = images;
            this.KontaktDetails = kontaktDetails.GetVisibleKontaktDetails(anzeige.KontaktOptionen);
        }

        public string toHTMLString()
        {
            try
            {
                string template = File.ReadAllText(Common.AppConfHelper.GetValueAsString("html_template_file"));
                template = template.Replace("\r\n", "");
                template = template.Replace("\"", "");
                template = template.Replace("%titel%", Titel);
                template = template.Replace("%beschreibung%", Beschreibung);
                template = template.Replace("%text%", Text);
                template = template.Replace("%kontaktdetails%", KontaktDetails.toHTMLString());
                string imageTemplate = Common.AppConfHelper.GetValueAsString("html_image_template");
                string images = "";
                foreach (var e in Images)
                {
                    images += imageTemplate.Replace("%imgpath%", e).Replace(@"\", "");
                }
                template = template.Replace("%images%", images);
                return template;
            }
            catch (Exception e)
            {
                Console.WriteLine("Fehler in toHTMLString:{0}", e.Message);
            }
            return "";
        }

        public override bool Equals(object obj)
        {
            var other = obj as TransportAnzeige;
            if (other == null)
                return false;
            return other.ID == this.ID;
        }

        public override int GetHashCode()
        {
            return ID;
        }
    }
}
