﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PersistenzManager;
using System.IO;
using System.Security.Cryptography;

namespace BilderKomponente
{
    public class BilderKomponente
    {
        private BilderVerwalter verwalter = null;

        public BilderKomponente(IPersistenz persistenz)
        {
            this.verwalter = new BilderVerwalter(persistenz);
        }

        public string GetVirtualImagePath(int imageID, int accountID)
        {
            Image img = verwalter.GetImage(imageID);
            if (img != null && img.AccountID.Equals(accountID))
                return img.VirtuellerPfad;
            return String.Empty;
        }

        public IList<string> GetVirtualImagePaths(IList<int> imageIDs, int accountID)
        {
            IList<string> paths = new List<string>();
            if (imageIDs == null)
                return paths;
            foreach (int i in imageIDs)
            {
                Image img = verwalter.GetImage(i);
                if (img != null && img.AccountID.Equals(accountID))
                    paths.Add(img.VirtuellerPfad);
            }
            return paths;
        }

        public IList<int> SelectOwnImages(IList<int> imageIDs, int accountID)
        {
            IList<int> images = new List<int>();
            if (imageIDs == null)
                return images;
            foreach (int i in imageIDs)
            {
                Image img = verwalter.GetImage(i);
                if (img != null && img.AccountID.Equals(accountID))
                    images.Add(img.ID);
            }
            return images;
        }

        public IList<string> GetVirtualImagePaths(IList<int> imageIDs)
        {
            IList<string> paths = new List<string>();
            if (imageIDs == null)
                return paths;
            foreach (int i in imageIDs)
            {
                Image img = verwalter.GetImage(i);
                if (img != null)
                    paths.Add(img.VirtuellerPfad);
            }
            return paths;
        }

        public string GetImageHash(int imageID, int accountID)
        {
            Image img = verwalter.GetImage(imageID);
            if (img != null && img.AccountID.Equals(accountID))
                return img.Hash;
            return String.Empty;
        }

        public int SaveImage(Stream imageStream, int accountID)
        {
            string dir = Common.AppConfHelper.GetValueAsString("image_directory");
            string virtualDir = Common.AppConfHelper.GetValueAsString("remote_image_directory");

            MemoryStream stream = new MemoryStream();

            byte[] buffer = new byte[1024];
            int read = 0;
            while ((read = imageStream.Read(buffer, 0, buffer.Length)) != 0)
                stream.Write(buffer, 0, read);
            stream.Seek(0, SeekOrigin.Begin);
            System.Security.Cryptography.SHA256 sha256 = System.Security.Cryptography.SHA256.Create();
            byte[] hashBytes = sha256.ComputeHash(stream);


            string sha256String = System.BitConverter.ToString(hashBytes);
            Image tmpImage = verwalter.GetImage(sha256String);
            if (tmpImage == null)
            {
                string filePath = dir + "\\" + sha256String + ".png";
                virtualDir += "/" + sha256String + ".png";
                FileStream outputStream = new FileStream(filePath, FileMode.Create);

                stream.Seek(0, SeekOrigin.Begin);
                buffer = new byte[1024];
                while ((read = stream.Read(buffer, 0, buffer.Length)) != 0)
                    outputStream.Write(buffer, 0, read);

                imageStream.Close();
                stream.Close();
                outputStream.Close();

                Image image = new Image(filePath, accountID, sha256String, virtualDir);
                verwalter.SaveImage(image);
                return image.ID;
            }
            else
                return tmpImage.ID;
        }
    }
}
