﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Service;
using Service.BoardService;

namespace CampusBoardWebsite
{
    public partial class Profil : AuthorizedContentWebPage
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            CheckSessionAndRedirect(true);
            try
            {
                var fassade = new Proxy();
                var profil = fassade.GetProfil(Guid);
                updateData(profil);
            }
            catch (Exception ex) { }
        }

        private void updateData(Service.BoardService.Profil profil)
        {
            if (profil == null)
                return;
            imgAvatar.ImageUrl = profil.Profilbild;
            txtNickname.Value = profil.Nickname;

            txtVorname.Value = profil.KontaktDetails.Vorname;
            txtNachname.Value = profil.KontaktDetails.Nachname;
            chkNameVisible.Checked = profil.KontaktDetails.NameVisible;

            txtStraße.Value = profil.KontaktDetails.Straße;
            txtPlz.Value = profil.KontaktDetails.Plz;
            txtStadt.Value = profil.KontaktDetails.Stadt;
            chkAddressVisible.Checked = profil.KontaktDetails.AdresseVisible;

            txtMail.Value = profil.KontaktDetails.Email;
            chkMailVisible.Checked = profil.KontaktDetails.EmailVisible;

            txtTelefon.Value = profil.KontaktDetails.Telefon;
            chkTelefonVisible.Checked = profil.KontaktDetails.TelefonVisible;

            chkSichtbar.Checked = profil.Sichtbar;
        }

        protected void btnSubmit_Click(object sender, EventArgs e)
        {
            var fassade = new Proxy();
            string nickname = Request.Params["ctl00$MainContent$txtNickname"];
            string vorname = Request.Params["ctl00$MainContent$txtVorname"];
            string nachname = Request.Params["ctl00$MainContent$txtNachname"];
            string straße = Request.Params["ctl00$MainContent$txtStraße"];
            string plz = Request.Params["ctl00$MainContent$txtPlz"];
            string stadt = Request.Params["ctl00$MainContent$txtStadt"];
            string email = Request.Params["ctl00$MainContent$txtMail"];
            string telefon = Request.Params["ctl00$MainContent$txtTelefon"];

            var chkName = Request.Params["ctl00$MainContent$chkNameVisible"];
            var chkAdresse = Request.Params["ctl00$MainContent$chkAddressVisible"];
            var chkMail = Request.Params["ctl00$MainContent$chkMailVisible"];
            var chkTelefon = Request.Params["ctl00$MainContent$chkTelefonVisible"];
            var chkSichtbar = Request.Params["ctl00$MainContent$chkSichtbar"];

            bool nameVisible = false, addressVisible = false, telefonVisible = false, mailVisible = false, sichtbar = false;

            if (!String.IsNullOrEmpty(chkName) && chkName.Equals("on"))
                nameVisible = true;
            if (!String.IsNullOrEmpty(chkAdresse) && chkAdresse.Equals("on"))
                addressVisible = true;
            if (!String.IsNullOrEmpty(chkMail) && chkMail.Equals("on"))
                mailVisible = true;
            if (!String.IsNullOrEmpty(chkTelefon) && chkTelefon.Equals("on"))
                telefonVisible = true;
            if (!String.IsNullOrEmpty(chkSichtbar) && chkSichtbar.Equals("on"))
                sichtbar = true;

            var profil = fassade.ProfilAktualisieren(Guid, nickname, vorname, nachname, nameVisible, straße, plz, stadt,
                    addressVisible, telefon, telefonVisible, email, mailVisible, -1, sichtbar);
            updateData(profil);
            if (profil == null)
            {
                alertError.Visible = true;
                alertSuccess.Visible = false;
                lblError.Text = "Beim Speichern ist ein Fehler aufgetreten.";
            }
            else
            {
                alertError.Visible = false;
                alertSuccess.Visible = true;
                lblSuccess.Text = "Profilinformationen erfolgreich geändert.";
            }
        }

        protected void btnDelete_Click(object sender, EventArgs e)
        {
            try
            {
                var proxy = new Proxy();
                var profil = proxy.GetProfil(Guid);
                profil = proxy.ProfilAktualisieren(Guid, profil.Nickname, profil.KontaktDetails.Vorname, profil.KontaktDetails.Nachname, profil.KontaktDetails.NameVisible,
                    profil.KontaktDetails.Straße, profil.KontaktDetails.Plz, profil.KontaktDetails.Stadt, profil.KontaktDetails.AdresseVisible, profil.KontaktDetails.Telefon,
                    profil.KontaktDetails.TelefonVisible, profil.KontaktDetails.Email, profil.KontaktDetails.EmailVisible, 0, profil.Sichtbar);
                updateData(profil);
            }
            catch (Exception ex) { }
        }

        protected void btnHochladen_Click(object sender, EventArgs e)
        {
            saveImages();
            var fassade = new Proxy();
            int id;
            Service.BoardService.Profil tmp = null;
            try
            {
                id = (int)Session["ProfilImageID"];
                var proxy = new Proxy();
                var profil = proxy.GetProfil(Guid);
                tmp = proxy.ProfilAktualisieren(Guid, profil.Nickname, profil.KontaktDetails.Vorname, profil.KontaktDetails.Nachname, profil.KontaktDetails.NameVisible,
                    profil.KontaktDetails.Straße, profil.KontaktDetails.Plz, profil.KontaktDetails.Stadt, profil.KontaktDetails.AdresseVisible, profil.KontaktDetails.Telefon,
                    profil.KontaktDetails.TelefonVisible, profil.KontaktDetails.Email, profil.KontaktDetails.EmailVisible, id, profil.Sichtbar);
                updateData(tmp);
            }
            catch (Exception ex) { }
            if (tmp == null)
            {
                alertError.Visible = true;
                alertSuccess.Visible = false;
                lblError.Text = "Hochladen fehlgeschlagen";
            }
            else
            {
                alertError.Visible = false;
                alertSuccess.Visible = true;
                lblSuccess.Text = "Dein Profilbild wurde erfolgreich hochgeladen!";
            }
            Session.Remove("ProfilImageID");
        }

        private void saveImages()
        {
            var fassade = new Proxy();
            foreach (var file in files.PostedFiles)
            {
                try
                {
                    string fileName = System.IO.Path.GetFileName(file.FileName);

                    string fileExtension = System.IO.Path.GetExtension(fileName).ToLower();

                    if (fileExtension.Equals(".jpg") || fileExtension.Equals(".png") || fileExtension.Equals(".bmp"))
                    {
                        Stream stream = file.InputStream;
                        FileTransferRequest request = new FileTransferRequest(Guid, stream);
                        FileTransferResult result = fassade.ImageErstellen(request);
                        if (result.imageID != -1)
                        {
                            Session.Add("ProfilImageID", result.imageID);
                        }
                    }
                }
                catch (Exception ex)
                {
                    alertError.Visible = true;
                    alertSuccess.Visible = false;
                    lblError.Text = "Hochladen fehlgeschlagen!";
                }

            }
        }
    }
}