﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Web;
using AnzeigenKomponente;
using ProfilKomponente;
using System.IO;
using FeedbackKomponente;

namespace Fassade
{
    [ServiceContract(Namespace = "http://marius-klausa.eu/board")]
    public interface IFassadeWebInterface
    {
        [OperationContract(Action = "Registrieren")]
        [WebInvoke(Method = "POST", UriTemplate = "Registrieren", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        string Registrieren(string email, string passwort);

        [OperationContract(Action = "Login")]
        [WebGet(UriTemplate = "Login?email={email}&passwort={passwort}", ResponseFormat = WebMessageFormat.Json)]
        string Login(string email, string passwort);

        [OperationContract(Action = "AccountAktivieren")]
        [WebInvoke(Method = "PUT", UriTemplate = "AccountAktivieren", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool AccountAktivieren(string guid, string code);

        [OperationContract(Action = "GetAccountMail")]
        [WebGet(UriTemplate = "GetAccountMail?guid={guid}", ResponseFormat = WebMessageFormat.Json)]
        string GetAccountMail(string guid);

        [OperationContract(Action = "SendeAktivierungsmail")]
        [WebInvoke(Method = "PUT", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool SendeAktivierungsmail(string guid);

        [OperationContract(Action = "IstAccountAktiviert")]
        [WebGet(UriTemplate = "IstAccountAktiviert?guid={guid}", ResponseFormat = WebMessageFormat.Json)]
        bool IstAccountAktiviert(string guid);

        [OperationContract(Action = "GetProfil")]
        [WebGet(UriTemplate = "GetProfil?guid={guid}", ResponseFormat = WebMessageFormat.Json)]
        Profil GetProfil(string guid);

        [OperationContract(Action = "GetProfilByID")]
        [WebGet(UriTemplate = "GetProfilByID?guid={guid}&profileAccountID={profileAccountID}", ResponseFormat = WebMessageFormat.Json)]
        Profil GetProfilByID(string guid, int profileAccountID);

        [OperationContract(Action = "GetProfilByRegisteredMail")]
        [WebGet(UriTemplate = "GetProfilByNickname?guid={guid}&registeredEmail={registeredEmail}", ResponseFormat = WebMessageFormat.Json)]
        Profil GetProfilByRegisteredMail(string guid, string registeredEmail);

        [OperationContract(Action = "GetAlleProfile")]
        [WebGet(UriTemplate = "GetAlleProfile?guid={guid}", ResponseFormat = WebMessageFormat.Json)]
        IList<ProfilVorschau> GetAlleProfile(string guid);

        [OperationContract(Action = "ProfilAktualisieren")]
        [WebInvoke(Method = "PUT", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        Profil ProfilAktualisieren(string guid, string nickname, string vorname, string nachname, bool nameVisible,
            string strasse, string plz, string stadt, bool adresseVisible, string telefon, bool telefonVisible,
            string email, bool emailVisible, int profilbildImageID, bool sichtbar);

        [OperationContract(Action = "AnzeigeErstellen")]
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        TransportAnzeige AnzeigeErstellen(string guid, string titel, string beschreibung, string text, IList<Tag> tags, IDictionary<int, int> kategorie, string appearance, IList<int> images, KontaktOptionen kontaktOptionen = KontaktOptionen.Öffentliche);

        [OperationContract(Action = "AnzeigeAktualisieren")]
        [WebInvoke(Method = "PUT", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        TransportAnzeige AnzeigeAktualisieren(string guid, int anzeigeID, string titel, string beschreibung, string text, IList<Tag> tags,
            IDictionary<int, int> kategorie, string appearance, IList<int> images);

        [OperationContract(Action = "AnzeigeSetÖffentlichStatus")]
        [WebInvoke(Method = "PUT", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool AnzeigeSetÖffentlichStatus(string guid, int anzeigeID, bool status);

        [OperationContract(Action = "AnzeigeLöschen")]
        [WebInvoke(Method = "DELETE", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool AnzeigeLöschen(string guid, int anzeigeID);

        [OperationContract(Action = "GetAllTags")]
        [WebGet(UriTemplate = "GetAllTags", ResponseFormat = WebMessageFormat.Json)]
        IList<Tag> GetAllTags();

        [OperationContract(Action = "GetAllKategorien")]
        [WebGet(UriTemplate = "GetAllKategorien", ResponseFormat = WebMessageFormat.Json)]
        IList<Kategorie> GetAllKategorien();

        [OperationContract(Action = "GetAllAnzeigen")]
        [WebGet(UriTemplate = "GetAllAnzeigen?guid={guid}&chunk={chunk}", ResponseFormat = WebMessageFormat.Json)]
        IList<TransportAnzeige> GetAllAnzeigen(string guid, int chunk);

        [OperationContract(Action = "GetOwnAnzeigen")]
        [WebGet(UriTemplate = "GetOwnAnzeigen?guid={guid}&chunk={chunk}", ResponseFormat = WebMessageFormat.Json)]
        IList<TransportAnzeige> GetOwnAnzeigen(string guid, int chunk);

        [OperationContract(Action = "GetAnzeige")]
        [WebGet(UriTemplate = "GetAnzeige?guid={guid}&anzeigeid={anzeigeid}", ResponseFormat = WebMessageFormat.Json)]
        TransportAnzeige GetAnzeige(string guid, int anzeigeID);

        [OperationContract(Action = "SendeAnzeigePerMail")]
        [WebGet(UriTemplate = "SendeAnzeigePerMail?guid={guid}&anzeigeid={anzeigeid}", ResponseFormat = WebMessageFormat.Json)]
        bool SendeAnzeigePerMail(string guid, int anzeigeID);

        [OperationContract(Action = "GetAllAnzeigenFromUser")]
        [WebGet(UriTemplate = "GetAllAnzeigenFromUser?guid={guid}&nickname={nickname}&chunk={chunk}", ResponseFormat = WebMessageFormat.Json)]
        IList<TransportAnzeige> GetAllAnzeigenFromUser(string guid, string nickname, int chunk);

        [OperationContract(Action = "GetImageHash")]
        [WebGet(UriTemplate = "GetImageHash?guid={guid}&imageid={imageid}", ResponseFormat = WebMessageFormat.Json)]
        string GetImageHash(string guid, int imageID);

        [OperationContract(Action = "GetImagePath")]
        [WebGet(UriTemplate = "GetImagePath?guid={guid}&imageid={imageid}", ResponseFormat = WebMessageFormat.Json)]
        string GetImagePath(string guid, int imageID);

        [OperationContract(Action = "GetImagePaths")]
        [WebInvoke(Method = "PUT", UriTemplate = "GetImagePaths?guid={guid}", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        IList<string> GetImagePaths(string guid, IList<int> imageids);

        [OperationContract(Action = "IstOwnAnzeige")]
        [WebGet(UriTemplate = "IstOwnAnzeige?guid={guid}&anzeigeID={anzeigeID}", ResponseFormat = WebMessageFormat.Json)]
        bool IstOwnAnzeige(string guid, int anzeigeID);

        [OperationContract(Action = "PostFeedback")]
        [WebInvoke(Method = "POST", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool PostFeedback(string guid, string text);

        [OperationContract(Action = "GetFeedbackList")]
        [WebGet(UriTemplate = "GetFeedbackList?guid={guid}&chunk={chunk}", ResponseFormat = WebMessageFormat.Json)]
        IList<Feedback> GetFeedbackList(string guid, int chunk);

        [OperationContract(Action = "Status")]
        [WebGet(UriTemplate = "Status", ResponseFormat = WebMessageFormat.Json)]
        bool Status();

        [OperationContract(Action = "ChangePassword")]
        [WebInvoke(Method = "POST", UriTemplate = "ChangePassword", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool ChangePassword(string guid, string guidToChange, string oldPW, string newPW);

        [OperationContract(Action = "DeleteKategorie")]
        [WebInvoke(Method = "POST", UriTemplate = "DeleteKategorie", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        bool DeleteKategorie(string guid, int kategorieID);

        [OperationContract(Action = "CreateKategorie")]
        [WebInvoke(Method = "POST", UriTemplate = "CreateKategorie", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        Kategorie CreateKategorie(string guid, string name, IList<string> unterkategorien);

        [OperationContract(Action = "GetGuid")]
        [WebInvoke(Method = "POST", UriTemplate = "GetGuid", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        string GetGuid(string guid, string email);

        [OperationContract(Action = "CreateAccount")]
        [WebInvoke(Method = "POST", UriTemplate = "CreateAccount", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        string CreateAccount(string guid, string email, string passwort, bool activated);

        [OperationContract(Action = "SendRundmail")]
        [WebInvoke(Method = "POST", UriTemplate = "SendRundmail", ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
        void SendRundmail(string guid, string subject, string text);
    }
}
