﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Controls;
using System.ComponentModel;
using Microsoft.Surface.Presentation.Controls;
using BasisKlassen;

namespace LoginRegisterApp
{
    public class Model : BaseModel, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        private LoginView loginView;
        private RegisterView registerView;
        private LogoutView logoutView;

        public Model(SurfaceWindow parent)
        {
            parent.Closing += SurfaceWindow_Closing;
            Views = new ObservableCollection<Control>();
            loginView = new LoginView(this);
            registerView = new RegisterView(this);
            logoutView = new LogoutView(this);
            views.Add(loginView);
            views.Add(registerView);
            //views.Add(logoutView);
        }
        private ObservableCollection<Control> views;

        public ObservableCollection<Control> Views
        {
            get
            {
                return views;
            }
            set
            {
                if (Views != value)
                {
                    views = value;
                    //this.OnPropertyChanged("Views");
                }
            }
        }

        public String Guid
        {
            get { return guid; }
            set
            {
                if (guid != value)
                {
                    guid = value;
                    this.OnPropertyChanged("Guid");
                    if (String.IsNullOrEmpty(guid))
                    {
                        views.Remove(logoutView);
                        views.Add(loginView);
                        views.Add(registerView);
                    }
                    else
                    {
                        views.Remove(loginView);
                        views.Remove(registerView);
                        views.Add(logoutView);
                    }
                }
            }
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SurfaceWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            loginView.IsActive = false;
        }
    }
}
