﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using AnzeigenApp;
using Common.Logger;
using Service;
using Service.BoardService;

namespace ProfilApp
{
    /// <summary>
    /// Interaktionslogik für ProfilDetailView.xaml
    /// </summary>
    public partial class ProfilDetailView : UserControl, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public ProfilDetailView()
        {
            InitializeComponent();
        }

        public ProfilDetailView(Profil profil)
        {
            InitializeComponent();
            updateData(profil);
            setNotVisible(profil);
        }

        public void update(Profil profil)
        {
            updateData(profil);
            setNotVisible(profil);
        }

        private void UserControl_DataContextChanged_1(object sender, DependencyPropertyChangedEventArgs e)
        {
            var profilVorschau = e.NewValue as ProfileHolder;
            try
            {
                var proxy = new Proxy();
                var profil = proxy.GetProfilByRegisteredMail(Guid, profilVorschau.RegisteredEmail);
                var ownProfil = proxy.GetProfil(Guid);
                updateData(profil);
                if (!profil.RegisteredEmail.Equals(ownProfil != null ? ownProfil.RegisteredEmail : ""))
                    setNotVisible(profil);
            }
            catch (Exception ex) { }
        }

        private void updateData(Profil profil)
        {
            try
            {
                Logger.LogEvent(LogEvent.ProfilAngeschaut, Guid);
                txtName.Text = (String.IsNullOrEmpty(profil.Nickname) ? profil.RegisteredEmail : profil.Nickname).ToUpper();
                imgAvatar.Source = BitmapLoader.GetImageSource(profil.Profilbild);
                txtVorname.Text = profil.KontaktDetails.Vorname;
                txtNachname.Text = profil.KontaktDetails.Nachname;
                txtStrasse.Text = profil.KontaktDetails.Straße;
                txtPlz.Text = profil.KontaktDetails.Plz;
                txtStadt.Text = profil.KontaktDetails.Stadt;
                txtEmail.Text = profil.KontaktDetails.Email;
                txtTelefon.Text = profil.KontaktDetails.Telefon;
            }
            catch (Exception ex) { }
        }

        private void setNotVisible(Profil profil)
        {
            try
            {
                if (!profil.KontaktDetails.NameVisible && !profil.KontaktDetails.AdresseVisible)
                    txtVorname.Text = "(nicht öffentlich)";
                if (!profil.KontaktDetails.AdresseVisible)
                    txtStrasse.Text = "(nicht öffentlich)";
                if (!profil.KontaktDetails.EmailVisible)
                    txtEmail.Text = "(nicht öffentlich)";
                if (!profil.KontaktDetails.TelefonVisible)
                    txtTelefon.Text = "(nicht öffentlich)";
            }
            catch (Exception ex) { }
        }

        public String Guid
        {
            get { return (String)this.GetValue(GuidProperty); }
            set { this.SetValue(GuidProperty, value); }
        }
        public static readonly DependencyProperty GuidProperty = DependencyProperty.Register(
          "Guid", typeof(String), typeof(ProfilDetailView), new PropertyMetadata(""));

        //private String guidToken;
        //public String GuidToken
        //{
        //    get { return guidToken; }
        //    set
        //    {
        //        if (GuidToken != value)
        //        {
        //            guidToken = value;
        //            this.OnPropertyChanged("GuidToken");
        //        }
        //    }
        //}

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
