﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common;
using PersistenzManager;
using RechteVerwaltungsKomponente;

namespace ProfilKomponente
{
    public class ProfilKomponente : IProfilKomponente
    {
        private ProfilVerwalter verwalter;

        public ProfilKomponente(IPersistenz persistenz)
        {
            verwalter = new ProfilVerwalter(persistenz);
        }

        public int GetAccountID(string nickname)
        {
            Profil p = verwalter.GetProfil(nickname);
            if (p != null)
                return p.AccountID;
            return -1;
        }

        public string GetNickname(int accountID)
        {
            return verwalter.GetNickname(accountID);
        }

        public Profil GetProfil(int accountID)
        {
            //eigenes profil lesen, darf man.
            return verwalter.GetProfil(accountID);
        }

        public Profil GetProfil(string registeredMail, int accountID, AccountType type)
        {
            //wenn eigenes profil, ist okay, sonst jenach accounttype, nur öffentliche stats oder alle
            Profil p = verwalter.GetProfil(registeredMail);
            if (p.AccountID.Equals(accountID))
                return p;
            switch (type)
            {
                case RechteVerwaltungsKomponente.AccountType.Admin: return p;
                default:
                    return p.Sichtbar ? p.GetVisibleProfile() : null;
            }
        }

        public IList<ProfilVorschau> GetAlleProfile(AccountType type)
        {
            return verwalter.GetAlleProfile(true).Select(p => new ProfilVorschau() { Displayname = String.IsNullOrEmpty(p.Nickname) ? p.RegisteredEmail : p.Nickname, Profilbild = p.Profilbild, RegisteredEmail = p.RegisteredEmail }).ToList();
            //switch (type)
            //{
            //    case RechteVerwaltungsKomponente.AccountType.Admin: return verwalter.GetAlleProfile(false).Select(p => new ProfilVorschau() { Displayname = String.IsNullOrEmpty(p.Nickname) ? p.RegisteredEmail : p.Nickname, Profilbild = p.Profilbild }).ToList();
            //    default: return verwalter.GetAlleProfile(true).Select(p => new ProfilVorschau() { Displayname = String.IsNullOrEmpty(p.Nickname) ? p.RegisteredEmail : p.Nickname, Profilbild = p.Profilbild }).ToList();
            //}
        }

        public Profil ProfilErstellen(int accountID, String registeredEmail)
        {
            Profil p = new Profil(accountID, registeredEmail);
            verwalter.SaveProfil(p);
            return p;
        }

        public Profil ProfilAktualisieren(int accountID, string nickname, string vorname, string nachname, bool nameVisible,
            string straße, string plz, string stadt, bool adresseVisible, string telefon, bool telefonVisibible,
            string email, bool emailVisible, string profilbildImage, bool sichtbar)
        {
            Profil p = verwalter.GetProfil(accountID);
            p.Nickname = nickname;
            p.KontaktDetails.Vorname = vorname;
            p.KontaktDetails.Nachname = nachname;
            p.KontaktDetails.NameVisible = nameVisible;
            p.KontaktDetails.Straße = straße;
            p.KontaktDetails.Plz = plz;
            p.KontaktDetails.Stadt = stadt;
            p.KontaktDetails.AdresseVisible = adresseVisible;
            p.KontaktDetails.Telefon = telefon;
            p.KontaktDetails.TelefonVisible = telefonVisibible;
            p.KontaktDetails.Email = email;
            p.KontaktDetails.EmailVisible = emailVisible;
            p.Sichtbar = sichtbar;
            if (!String.IsNullOrEmpty(profilbildImage) && !profilbildImage.Equals("keep"))
                p.Profilbild = profilbildImage;
            else if (profilbildImage.Equals("keep"))
                p.Profilbild = p.Profilbild;
            else
                p.Profilbild = AppConfHelper.GetValueAsString("avatar_placeholder");
            verwalter.SaveProfil(p);
            return p;
        }

        public Profil GetProfil(int profileAccountID, int userAccountID, AccountType type)
        {
            //wenn eigenes profil, ist okay, sonst jenach accounttype, nur öffentliche stats oder alle
            Profil p = verwalter.GetProfil(profileAccountID);
            if (p.AccountID.Equals(userAccountID))
                return p;
            switch (type)
            {
                case RechteVerwaltungsKomponente.AccountType.Admin: return p;
                default:
                    {
                        return p.Sichtbar ? p.GetVisibleProfile() : null;
                    };
            }
        }


        public Profil ChangeToSichtbar(int accountID)
        {
            var profil = verwalter.GetProfil(accountID);
            profil.Sichtbar = true;
            verwalter.SaveProfil(profil);
            return profil;
        }
    }
}
