﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using Common;
using PersistenzManager;

namespace RechteVerwaltungsKomponente
{
    public class RechteVerwaltung
    {
        private IPersistenz Persistenz = null;

        public RechteVerwaltung(IPersistenz persistenz)
        {
            this.Persistenz = persistenz;
        }

        public int GetAccountID(string email)
        {
            Account account = (from Account u in Persistenz.LinqQuery<Account>()
                               where (u.Email.Equals(email))
                               select u).SingleOrDefault();
            if (account != null)
                return account.ID;
            return -1;
        }

        public string Login(string email, string passwort)
        {
            //Account userRights = (from Account u in Persistenz.GetAll<Account>()
            //                      where ((u.Email.Equals(email)) && (u.Passwort.Equals(passwort)))
            //                      select u).SingleOrDefault();
            var hasher = new SHA256Managed();
            var coder = new System.Text.UTF8Encoding();
            passwort = BitConverter.ToString(hasher.ComputeHash(coder.GetBytes(passwort))).Replace("-","");
            Account userRights = (from Account u in Persistenz.LinqQuery<Account>()
                                  where ((u.Email.Equals(email)) && (u.Passwort.Equals(passwort)))
                                  select u).SingleOrDefault();
            if (userRights == null)
                return string.Empty;
            return userRights.Guid;
        }

        //public bool IsValidTicket(string guid)
        //{
        //    if (guid != null)
        //        return SitzungsVerwaltung.CheckSession(ticket.Sitzung, ticket.UserId);
        //    return false;
        //}

        public bool IstAccountAktiviert(string guid)
        {
            Account userRights = GetAccount(guid);
            if (userRights == null)
                return false;
            if (userRights.Status >= AccountType.Aktiv)
                return true;
            return false;
        }

        public string GetAccountMail(string guid)
        {
            Account account = GetAccount(guid);
            if (account != null)
                return account.Email;
            return String.Empty;
        }

        private Account GetAccount(string guid)
        {
            return (from Account u in Persistenz.LinqQuery<Account>()
                    where (u.Guid.Equals(guid))
                    select u).SingleOrDefault();
        }

        public string GetGuidFromAccount(string email, string passwort)
        {
            Account account = (from Account u in Persistenz.LinqQuery<Account>()
                               where ((u.Email.Equals(email)) && (u.Passwort.Equals(passwort)))
                               select u).SingleOrDefault();
            if (account != null)
                return account.Guid;
            return string.Empty;
        }

        public string GetGuidFromAccount(string email)
        {
            email = email.ToLower();
            Account account = (from Account u in Persistenz.LinqQuery<Account>()
                               where (u.Email.Equals(email))
                               select u).SingleOrDefault();
            if (account != null)
                return account.Guid;
            return string.Empty;
        }

        public string Register(string email, string passwort, bool activated, bool isAdmin, out int accountID)
        {
            Account userRights;
            if (!isAdmin && !checkEmail(email))
            {
                accountID = -1;
                return string.Empty;
            }
            if (!activated)
                userRights = new Account(email, passwort, AccountType.Inaktiv, Guid.NewGuid());
            else
                userRights = new Account(email, passwort, AccountType.Aktiv, Guid.NewGuid());
            try
            {
                Persistenz.Save(userRights);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                accountID = -1;
                return string.Empty;
            }
            accountID = userRights.ID;
            return userRights.Guid;
        }

        private bool checkEmail(string mail)
        {
            if (AppConfHelper.GetValueAsBool("allow_everything"))
                return true;
            Console.WriteLine("CheckEmail for {0}:", mail);
            if (String.IsNullOrEmpty(mail))
            {
                Console.WriteLine("IsNullOrEmpty:True");
                return false;
            }
            if (mail.Count(x => x.Equals('@')) != 1)
            {
                Console.WriteLine("Count of \"@\" != 1");
                return false;
            }
            if (AppConfHelper.GetValueAsBool("allow_haw_domain_only"))
                if (!mail.Substring(mail.IndexOf("@")).ToLower().Equals("@haw-hamburg.de"))
                {
                    Console.WriteLine("Domain != @haw-hamburg.de");
                    return false;
                }
            Console.WriteLine("everything is fine");
            return true;
        }

        public int IsAccessPermitted(Anwendungsfall anwendungsfall, string guid)
        {
            //string rechte = SitzungsVerwaltung.GetSessionRights(ticket.Sitzung, ticket.UserId);
            //AccountType status = (AccountType)Enum.Parse(typeof(AccountType), rechte);
            Account userRights = (from Account u in Persistenz.LinqQuery<Account>()
                                  where (u.Guid.Equals(guid))
                                  select u).SingleOrDefault();
            if (userRights == null)
                return -1;
            AccountType methodAccess = GetMethodAccess(anwendungsfall);
            if (userRights.Status >= methodAccess)
                return userRights.ID;
            return -1;
        }

        public int IsAccessPermitted(Anwendungsfall anwendungsfall, string guid, out AccountType type)
        {
            //string rechte = SitzungsVerwaltung.GetSessionRights(ticket.Sitzung, ticket.UserId);
            //AccountType status = (AccountType)Enum.Parse(typeof(AccountType), rechte);
            Account userRights = (from Account u in Persistenz.LinqQuery<Account>()
                                  where (u.Guid.Equals(guid))
                                  select u).SingleOrDefault();
            if (userRights == null)
            {
                type = AccountType.None;
                return -1;
            }
            type = userRights.Status;
            AccountType methodAccess = GetMethodAccess(anwendungsfall);
            if (userRights.Status >= methodAccess)
            {
                return userRights.ID;
            }
            return -1;
        }

        private AccountType GetMethodAccess(Anwendungsfall anwendungsfall)
        {
            //IList<Berechtigung> ll = Persistenz.GetAll<Berechtigung>();
            //foreach (Berechtigung b in ll)
            //    if (b.Anwendungsfall.Equals(anwendungsfall))
            //        return b.Status;
            //return 0;
            return (from Berechtigung ber in Persistenz.LinqQuery<Berechtigung>()
                    where (ber.Anwendungsfall == anwendungsfall)
                    select ber.Status).SingleOrDefault();
        }

        public void ChangeStatus(int accountID, AccountType status)
        {
            Account rechte = (from Account recht in Persistenz.LinqQuery<Account>()
                              where (recht.ID == accountID)
                              select recht).SingleOrDefault();
            if (rechte == null)
                return;
            //SitzungsVerwaltung.NewSession(rechte.ID, status.ToString());//Muss das?
            rechte.Status = status;
            Persistenz.Save(rechte);
        }

        public bool ChangePassword(int accountID, string oldPW, string newPW, AccountType type)
        {
            var hasher = new SHA256Managed();
            var coder = new System.Text.UTF8Encoding();
            oldPW = BitConverter.ToString(hasher.ComputeHash(coder.GetBytes(oldPW))).Replace("-", "");
            newPW = BitConverter.ToString(hasher.ComputeHash(coder.GetBytes(newPW))).Replace("-", "");
            Account userRights;
            switch (type)
            {
                case AccountType.Admin: userRights = (from Account u in Persistenz.LinqQuery<Account>()
                                                      where (u.ID == accountID)
                                                      select u).SingleOrDefault(); break;
                case AccountType.Aktiv: userRights = (from Account u in Persistenz.LinqQuery<Account>()
                                                      where ((u.ID == accountID)) && (u.Passwort.Equals(oldPW))
                                                      select u).SingleOrDefault(); break;
                default: return false;
            }

            if (userRights == null)
                return false;
            userRights.Passwort = newPW;
            Persistenz.Save<Account>(userRights);
            return true;
        }

        //public bool Logout(Ticket ticket)
        //{
        //    string rechte = SitzungsVerwaltung.GetSessionRights(ticket.Sitzung, ticket.UserId);
        //    AccountType status = (AccountType)Enum.Parse(typeof(AccountType), rechte);
        //    if (status >= AccountType.None)
        //    {
        //        SitzungsVerwaltung.RemoveSession(ticket.UserId);
        //        return true;
        //    }
        //    return false;
        //}
    }
}
