﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RechteVerwaltungsKomponente;
using Common;
using PersistenzManager;
using System.Security.Cryptography;

namespace AccountVerwaltungsKomponente
{
    public class AccountVerwaltung
    {
        private IPersistenz persistenz = null;
        private RechteVerwaltung rechteVerwaltung = null;

        public AccountVerwaltung(IPersistenz persistenz, RechteVerwaltung rechteVerwaltung)
        {
            this.persistenz = persistenz;
            this.rechteVerwaltung = rechteVerwaltung;
        }

        //public string Registrieren(string email, string passwort)
        //{
        //    int id;
        //    bool exists;
        //    return Registrieren(email, passwort, out id, out exists);
        //}

        public string Registrieren(string email, string passwort, bool activated, bool isAdmin, out int id, out bool exists)
        {
            var hasher = new SHA256Managed();
            var coder = new System.Text.UTF8Encoding();
            passwort = BitConverter.ToString(hasher.ComputeHash(coder.GetBytes(passwort))).Replace("-", "");
            string guid = rechteVerwaltung.GetGuidFromAccount(email, passwort);
            if (guid.Length > 0)
            {
                id = rechteVerwaltung.GetAccountID(email);
                exists = true;
                return guid;
            }
            exists = false;
            return Register(email, passwort, activated, isAdmin, out id);
        }

        private string Register(string mailAdress, string passwort, bool activated, bool isAdmin, out int id)
        {
            int accountID;
            string guid = rechteVerwaltung.Register(mailAdress, passwort, activated, isAdmin, out accountID);
            id = accountID;
            if (guid.Length > 0)
            {
                Aktivierung tmp = new Aktivierung(GetRandomCode(), mailAdress, accountID);
                persistenz.Save(tmp);
                if (!activated)
                    SendeAktivierungsmail(accountID, guid);
                return guid;
            }
            return string.Empty;
        }

        private string GetRandomCode()
        {
            return new Random().Next(1000000, 9999999).ToString();
        }

        public bool SendeAktivierungsmail(int accountID, string guid)
        {
            Aktivierung aktivierung = (from Aktivierung act in persistenz.LinqQuery<Aktivierung>()
                                       where (act.AccountID == accountID)
                                       select act).SingleOrDefault();
            if (aktivierung == null)
                return false;
            if (AppConfHelper.GetValueAsBool("aktivierungsmail"))
            {
                string text = "Unter folgendem Link kannst du deinen Account aktivieren: http://campus-board.marius-klausa.eu/Aktivieren.aspx?guid=" + guid + "&code=" + aktivierung.Aktivierungscode;
                return MailKomponente.MailKomponente.SendMail(aktivierung.MailAdress, "Aktivierungs Code", text);
            }
            else
                return true;
        }

        public bool Aktivieren(int accountID, string code)
        {
            Aktivierung aktivierung = (from Aktivierung act in persistenz.LinqQuery<Aktivierung>()
                                       where (act.AccountID == accountID)
                                       select act).SingleOrDefault();
            //            IList<Aktivierung> ll = persistenz.GetAll<Aktivierung>();
            //foreach(Aktivierung a in ll)
            //    if(a.ID.Equals(id)
            if (aktivierung == null)//hinzugefügt
                return true;
            if (aktivierung.Aktivierungscode.Equals(code))
            {
                rechteVerwaltung.ChangeStatus(accountID, AccountType.Aktiv);
                persistenz.Delete(aktivierung);
                return true;
            }
            else
                return false;
        }

        public void SendRundmail(string subject, string text)
        {
            foreach (var acc in persistenz.GetAll<Account>())
                MailKomponente.MailKomponente.SendMail(acc.Email, subject, text);
        }
    }
}
