﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Service.BoardService;
using System.ComponentModel;

namespace Service
{
    public class Anzeige : TransportAnzeige, INotifyPropertyChanged
    {
        public new event PropertyChangedEventHandler PropertyChanged;
        public new int ID { get; private set; }
        public new int AccountID { get; private set; }
        public new string profilNickname { get; private set; }
        public new String Titel { get; set; }
        public new String Beschreibung { get; set; }
        public new String Text { get; set; }
        public new DateTime Datum { get; set; }
        public new TimeSpan Restzeit { get; set; }
        public new KontaktDetails KontaktDetails { get; set; }
        public new IList<string> Tags { get; set; }
        public new IDictionary<int, string> Kategorie { get; set; }
        public new string Appearance { get; set; }
        public new IList<string> Images { get; set; }
        private bool published;
        public new bool Published
        {
            get { return published; }
            set
            {
                if (published != value)
                {
                    published = value;
                    OnPropertyChanged("Published");
                }
            }
        }

        public void UpdateValues(Anzeige anzeige)
        {
            this.Titel = anzeige.Titel;
            this.ID = anzeige.ID;
            this.AccountID = anzeige.AccountID;
            this.profilNickname = anzeige.profilNickname;
            this.Beschreibung = anzeige.Beschreibung;
            this.Text = anzeige.Text;
            this.Datum = anzeige.Datum;
            this.Restzeit = anzeige.Restzeit;
            this.KontaktDetails = anzeige.KontaktDetails;
            this.Published = anzeige.Published;
            this.Tags = anzeige.Tags;
            this.Kategorie = anzeige.Kategorie;
            this.Appearance = anzeige.Appearance;
            this.Images = anzeige.Images;
        }

        public Anzeige() { }

        public Anzeige(TransportAnzeige anzeige)
        {
            if (anzeige != null)
            {
                this.Titel = anzeige.Titel;
                this.ID = anzeige.ID;
                this.AccountID = anzeige.AccountID;
                this.profilNickname = anzeige.profilNickname;
                this.Beschreibung = anzeige.Beschreibung;
                this.Text = anzeige.Text;
                this.Datum = anzeige.Datum;
                this.Restzeit = anzeige.Restzeit;
                this.KontaktDetails = anzeige.KontaktDetails;
                this.Published = anzeige.Published;
                this.Tags = anzeige.Tags;
                this.Kategorie = anzeige.Kategorie;
                this.Appearance = anzeige.Appearance;
                this.Images = anzeige.Images;
            }
        }

        public override bool Equals(object obj)
        {
            var other = obj as Anzeige;
            if (other == null)
                return false;
            return other.ID == this.ID;
        }

        public override int GetHashCode()
        {
            return ID;
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public override string ToString()
        {
            return Titel + Beschreibung + Text + ID;
        }

        public String ToJsonString(int maxSize)
        {
            string template = "{{\"ID\":{0},\"Titel\":\"{1}\",\"Beschreibung\":\"{2}\",\"Text\":\"{3}\",\"KontaktDetails\":{4}}}";
            int size = template.Length - 14;

            int titleRestLength = maxSize - ID.ToString().Length - size;
            string title = "";
            if (titleRestLength > Titel.Length)
                title = Titel.Substring(0, Titel.Length);
            else if (titleRestLength > 0)
                title = Titel.Substring(0, titleRestLength);

            int descriptionRestLength = maxSize - ID.ToString().Length - size - title.Length;
            string description = "";
            if (Beschreibung != null)
            {
                if (descriptionRestLength > Beschreibung.Length)
                    description = Beschreibung.Substring(0, Beschreibung.Length);
                else if (descriptionRestLength > 0)
                    description = Beschreibung.Substring(0, descriptionRestLength);
            }

            int textRestLength = maxSize - ID.ToString().Length - size - title.Length - description.Length;
            string text = "";
            if (textRestLength > Text.Length)
                text = Text.Substring(0, Text.Length);
            else if (textRestLength > 0)
                text = Text.Substring(0, textRestLength);
            String kontakt = "";
            //int length = size + title.Length + description.Length + text.Length + ID.ToString().Length;
            if (KontaktDetails != null)
                kontakt = "{\"Vorname\":\"" + (KontaktDetails.Vorname != null ? KontaktDetails.Vorname.Replace("\"", "") : "") + "\"," +
                    "\"Nachname\":\" " + (KontaktDetails.Nachname != null ? KontaktDetails.Nachname.Replace("\"", "") : "") + "\"," +
                    "\"Straße\":\"" + (KontaktDetails.Straße != null ? KontaktDetails.Straße.Replace("\"", "") : "") + "\"," +
                    "\"Plz\":\"" + (KontaktDetails.Plz != null ? KontaktDetails.Plz.Replace("\"", "") : "") + "\"," +
                    "\"Stadt\":\"" + (KontaktDetails.Stadt != null ? KontaktDetails.Stadt.Replace("\"", "") : "") + "\"," +
                    "\"Telefon\":\"" + (KontaktDetails.Telefon != null ? KontaktDetails.Telefon.Replace("\"", "") : "") + "\"," +
                    "\"Email\":\"" + (KontaktDetails.Email != null ? KontaktDetails.Email.Replace("\"", "") : "") + "\"}";
            return String.Format(template, ID, title.Replace("\"", ""), description.Replace("\"", ""), text.Replace("\"", ""), kontakt);
        }
    }
}
