﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.ServiceModel;

namespace SitzungsVerwaltungsKomponente
{
    public class SitzungsVerwaltung
    {
        private static IDictionary<int, SitzungTupel> Sessions = new Dictionary<int, SitzungTupel>();
        private static TimeSpan Timeout = new TimeSpan(0, 20, 0);
        private static readonly Object myLock = new Object();

        //static IClient callback = null;

        public static void Start()
        {
            new Task(() => Run()).Start();
        }

        public static string GetSessionRights(SitzungsID sessionID, int userId)
        {
            SitzungTupel tupel = null;
            lock (myLock)
            {
                Sessions.TryGetValue(userId, out tupel);
            }
            if (tupel != null)
                return tupel.Status;
            return "None";
        }

        public static bool CheckSession(SitzungsID sessionID, int userId)
        {
            SitzungTupel tupel = null;
            //if (userId == null)
            //    return false;
            lock (myLock)
            {
                Sessions.TryGetValue(userId, out tupel);
            }
            if (tupel != null)
                return (tupel.SessionID.Equals(sessionID));
            return false;
        }

        public static Guid NewSession(int userId, string status)
        {
            Console.WriteLine("Neue Sitzung für {0}, als {1}", userId, status);
            SitzungTupel tupel = null;
            lock (myLock)
            {
                Sessions.TryGetValue(userId, out tupel);
            }
            if (tupel != null)
            {
                tupel.Status = status;
                return tupel.SessionID.ID;
            }
            tupel = new SitzungTupel(new SitzungsID(Guid.NewGuid()), status, DateTime.Now);
            lock (myLock)
            {
                Sessions.Add(userId, tupel);
            }
            return tupel.SessionID.ID;
        }

        public static void RemoveSession(int userId)
        {
            Console.WriteLine("Entferne Sitzung für {0}", userId);
            lock (myLock)
            {
                Sessions.Remove(userId);
            }
        }

        private static void Run()
        {
            while (true)
            {
                Console.WriteLine("Sessionverwaltung läuft");
                Thread.Sleep(new TimeSpan(0, 0, 5));
                IList<int> rememberToDelete = new List<int>();
                lock (myLock)
                {
                    foreach (KeyValuePair<int, SitzungTupel> kvp in Sessions)
                    {
                        //kvp.Value.Callback.SendMessage("session läuft noch ");
                        if ((DateTime.Now - kvp.Value.Timestamp) > Timeout)
                        {
                            Console.WriteLine("session für {0}, sitzungsdauer:{1} delete", kvp.Key, (DateTime.Now - kvp.Value.Timestamp).ToString());
                            rememberToDelete.Add(kvp.Key);
                        }
                        else
                            Console.WriteLine("session für {0}, sitzungsdauer:{1} ok", kvp.Key, (DateTime.Now - kvp.Value.Timestamp).ToString());
                    }
                    foreach (int s in rememberToDelete)
                        Sessions.Remove(s);
                }
            }
        }
    }
}
